

#R LIBRARIES----------------------------------------------------------------------------------------------------------------------------------------
.libPaths("C:/R/library") # load R packages: the path has to be adjusted to where the packages are saved
library('dplyr')
library('zipfR')
library("ggplot2")


#Import dataset-----------------------------------------------------------------------------------------------------------------------------------

final_dataset<-read.table(file=file.choose(), header=TRUE, sep=";", comment.char="", fill=TRUE, na.strings = "character", quote="", row.names=NULL, stringsAsFactors=FALSE, strip.white=TRUE, encoding = "utf-8", blank.lines.skip = TRUE) # load the data file "02_dataset_semi_copula_raken_DEF.csv"


#Factors------------------------------------------------------------------------------------------------------------------------------------------

final_dataset$SUBJECT_COMPLEMENT_TYPE<-as.factor(final_dataset$SUBJECT_COMPLEMENT_TYPE)


#Type-Frequency List for different sample sizes---------------------------------------------------------------------------------------------------

data_input<-final_dataset$SUBJECT_COMPLEMENT_TYPE

sample_sizes_plot<-list()

sample_sizes_plot_1<-data_input[1:250]
sample_sizes_plot_1<-as.data.frame(table(sample_sizes_plot_1))
colnames(sample_sizes_plot_1)<-c("SUBJECT_COMPLEMENT_TYPE","Freq")
sample_sizes_plot_2<-data_input[1:500]
sample_sizes_plot_2<-as.data.frame(table(sample_sizes_plot_2))
colnames(sample_sizes_plot_2)<-c("SUBJECT_COMPLEMENT_TYPE","Freq")
sample_sizes_plot_3<-data_input[1:750]
sample_sizes_plot_3<-as.data.frame(table(sample_sizes_plot_3))
colnames(sample_sizes_plot_3)<-c("SUBJECT_COMPLEMENT_TYPE","Freq")
sample_sizes_plot_4<-data_input[1:1000]
sample_sizes_plot_4<-as.data.frame(table(sample_sizes_plot_4))
colnames(sample_sizes_plot_4)<-c("SUBJECT_COMPLEMENT_TYPE","Freq")
sample_sizes_plot_5<-data_input[1:1250]
sample_sizes_plot_5<-as.data.frame(table(sample_sizes_plot_5))
colnames(sample_sizes_plot_5)<-c("SUBJECT_COMPLEMENT_TYPE","Freq")
sample_sizes_plot_6<-data_input[1:1500]
sample_sizes_plot_6<-as.data.frame(table(sample_sizes_plot_6))
colnames(sample_sizes_plot_6)<-c("SUBJECT_COMPLEMENT_TYPE","Freq")

sample_sizes_plot_total<-rbind(sample_sizes_plot_1, sample_sizes_plot_2, sample_sizes_plot_3, sample_sizes_plot_4, sample_sizes_plot_5, sample_sizes_plot_6)
sample_sizes_plot_total["VERB"]<-c(rep("RAKEN_1", length(sample_sizes_plot_1[,1])), rep("RAKEN_2", length(sample_sizes_plot_2[,1])), rep("RAKEN_3", length(sample_sizes_plot_3[,1])), rep("RAKEN_4", length(sample_sizes_plot_4[,1])), rep("RAKEN_5", length(sample_sizes_plot_5[,1])), rep("RAKEN_6", length(sample_sizes_plot_6[,1])))
type_dataframe<-filter(sample_sizes_plot_total, Freq!=0)
ww<-as.character(unique(type_dataframe[,"VERB"]))

list_dataframes<-list()   # Type-Frequency List (Zipf ranking)
for (i in 1:length(ww)) { 
  a<-filter(type_dataframe, VERB == ww[i])
  f<-a[,"Freq"]
  type_freq<-tfl(f, k=1:length(f))
  list_dataframes[[i]]<-type_freq}

list_dataframes_spec<-list()   # Frequency Spectrum
for (i in 1:length(ww)) {                     
  freq_spec<-tfl2spc(list_dataframes[[i]])
  list_dataframes_spec[[i]]<-freq_spec}

# cf. Figure 4 in the associated article 
png(file = "Type-Frequency_List_for_different_sample_sizes.png") # save plot
type_freq_2<-plot(list_dataframes[[1]], list_dataframes[[2]], list_dataframes[[3]], list_dataframes[[4]], list_dataframes[[5]], list_dataframes[[6]], bw=TRUE, col=c("#e41a1c", "#377eb8", "#4daf4a", "#984ea3", "#ff7f00", "#ffff33", "#a65628") , legend=ww, xlim=c(0,100), ylim=c(0,120), type="b")
dev.off()



# Empirical vocabulary growth curve with V1-------------------------------------------------------------------------------------------------------

c<-vector() # type frequency

  for (j in 1:length(final_dataset[,1])) {
    b<-final_dataset$SUBJECT_COMPLEMENT_TYPE[1:j]
    c<-append(c, length(unique(b)))
  } 
  
  d<-c(1:length(final_dataset[,1]))
  empirical_vgc<-vgc(N=d, V = c)
  
z<-vector() # V1

for (j in 1:length(final_dataset[,1])) {
  b1<-final_dataset$SUBJECT_COMPLEMENT_TYPE[1:j]
  b2<-as.data.frame(table(b1))
  b3<-filter(b2, Freq==1)
  b4<-nrow(b3)
  z<-append(z, b4)
} 
  
d<-c(1:length(final_dataset[,1]))
empirical_vgc_V1<-vgc(N=d, V = empirical_vgc$V, Vm=data.frame(z))
  

# Hapax type ratio, hapax token ratio and type token ratio----------------------------------------------------------------------------------------

raken_hapax_type_ratio<-Vm(empirical_vgc_V1,1) / V(empirical_vgc_V1)  # calculate three ratio measures
N<-c(1:1500)
raken_hapax_token_ratio<-Vm(empirical_vgc_V1,1) / N
raken_type_token_ratio<- V(empirical_vgc_V1) / N

N_2<-rep(N, 3)   # create a dataframe with three ratio measures
ratio_measures<-rep(c("hapax_type_ratio", "hapax_token_ratio", "type_token_ratio"), each= 1500)
raken_overview<- c(raken_hapax_type_ratio, raken_hapax_token_ratio, raken_type_token_ratio)
raken_data_frame<-data.frame(N_2, ratio_measures, raken_overview)

# cf. Figure 1 in the associated article 
png(file = "hapax_type_ratio_and_other_ratios.png") # save plot
ggplot(data = raken_data_frame, aes(x=N_2, y= raken_overview)) + geom_point(shape=1, aes(colour=ratio_measures)) + theme_minimal() + xlab("sample size") + ylab("ratio values") +  scale_color_manual(name="Ratios", labels=c("hapax token ratio","hapax type ratio","type token ratio"),           values=c("#e41a1c", "#377eb8", "#4daf4a"))                  
dev.off()                   


# Basic measures----------------------------------------------------------------------------------------------------------------------------------

data_input<-final_dataset$SUBJECT_COMPLEMENT_TYPE

hapax_plot<-list()
hapax_plot[[1]]<-data_input[1:250]
hapax_plot[[1]]<-as.data.frame(table(hapax_plot[[1]]))
hapax_plot[[1]]<-filter(hapax_plot[[1]], Freq==1)
hapax_plot[[2]]<-data_input[1:500]
hapax_plot[[2]]<-as.data.frame(table(hapax_plot[[2]]))
hapax_plot[[2]]<-filter(hapax_plot[[2]], Freq==1)
hapax_plot[[3]]<-data_input[1:750]
hapax_plot[[3]]<-as.data.frame(table(hapax_plot[[3]]))
hapax_plot[[3]]<-filter(hapax_plot[[3]], Freq==1)
hapax_plot[[4]]<-data_input[1:1000]
hapax_plot[[4]]<-as.data.frame(table(hapax_plot[[4]]))
hapax_plot[[4]]<-filter(hapax_plot[[4]], Freq==1)
hapax_plot[[5]]<-data_input[1:1250]
hapax_plot[[5]]<-as.data.frame(table(hapax_plot[[5]]))
hapax_plot[[5]]<-filter(hapax_plot[[5]], Freq==1)
hapax_plot[[6]]<-data_input[1:1500]
hapax_plot[[6]]<-as.data.frame(table(hapax_plot[[6]]))
hapax_plot[[6]]<-filter(hapax_plot[[6]], Freq==1)



number_of_hapaxes<-vector()
for(i in 1:6) {
  number_of_hapaxes<-append(number_of_hapaxes, length(hapax_plot[[i]]$Var1))
}

non_hapax_plot<-list()
non_hapax_plot[[1]]<-data_input[1:250]
non_hapax_plot[[1]]<-as.data.frame(table(non_hapax_plot[[1]]))
non_hapax_plot[[1]]<-filter(non_hapax_plot[[1]], Freq!=0 & Freq!=1)
non_hapax_plot[[2]]<-data_input[1:500]
non_hapax_plot[[2]]<-as.data.frame(table(non_hapax_plot[[2]]))
non_hapax_plot[[2]]<-filter(non_hapax_plot[[2]], Freq!=0 & Freq!=1)
non_hapax_plot[[3]]<-data_input[1:750]
non_hapax_plot[[3]]<-as.data.frame(table(non_hapax_plot[[3]]))
non_hapax_plot[[3]]<-filter(non_hapax_plot[[3]], Freq!=0 & Freq!=1)
non_hapax_plot[[4]]<-data_input[1:1000]
non_hapax_plot[[4]]<-as.data.frame(table(non_hapax_plot[[4]]))
non_hapax_plot[[4]]<-filter(non_hapax_plot[[4]], Freq!=0 & Freq!=1)
non_hapax_plot[[5]]<-data_input[1:1250]
non_hapax_plot[[5]]<-as.data.frame(table(non_hapax_plot[[5]]))
non_hapax_plot[[5]]<-filter(non_hapax_plot[[5]], Freq!=0 & Freq!=1)
non_hapax_plot[[6]]<-data_input[1:1500]
non_hapax_plot[[6]]<-as.data.frame(table(non_hapax_plot[[6]]))
non_hapax_plot[[6]]<-filter(non_hapax_plot[[6]], Freq!=0 & Freq!=1)


number_of_hapaxes<-vector()
number_of_non_hapaxes<-vector()
number_of_non_hapaxes_tokens<-vector()
temporary_type_token_10<-vector()
temporary_type_token_90<-vector()
type_frequency_10<-vector()
type_frequency_90<-vector()
token_frequency_10<-vector()
token_frequency_90<-vector()

for(i in 1:6) {
  number_of_hapaxes<-append(number_of_hapaxes, length(hapax_plot[[i]]$Var1))
  number_of_non_hapaxes<-append(number_of_non_hapaxes, length(non_hapax_plot[[i]]$Var1)) # non-hapax types
  number_of_non_hapaxes_tokens<-append(number_of_non_hapaxes_tokens, sum(non_hapax_plot[[i]]$Freq)) # non-hapax tokens
  
  temporary<-non_hapax_plot[[i]]
  
  temporary_10 <- temporary[temporary$Freq > quantile(temporary$Freq, 0.9),]
  temporary_type_token_10 <- append(temporary_type_token_10, length(temporary_10$Var1) / sum(temporary_10$Freq))
  type_frequency_10<-append(type_frequency_10, length(temporary_10$Var1))
  token_frequency_10<-append(token_frequency_10, sum(temporary_10$Freq))
  temporary_90 <- temporary[temporary$Freq <= quantile(temporary$Freq, 0.9),]
  temporary_type_token_90 <- append(temporary_type_token_90, length(temporary_90$Var1) / sum(temporary_90$Freq))
  type_frequency_90<-append(type_frequency_90, length(temporary_90$Var1))
  token_frequency_90<-append(token_frequency_90, sum(temporary_90$Freq))
}


var_10_mean<-vector()
var_20_mean<-vector()

for (i in 1:6) {
  
  var_10_mean_temporary<-non_hapax_plot[[i]]
  var_10_mean_temporary_2<- var_10_mean_temporary[var_10_mean_temporary$Freq > quantile(var_10_mean_temporary$Freq, 0.9),]
  var_10_mean <- append(var_10_mean, mean(var_10_mean_temporary_2$Freq))
  
  var_20_mean_temporary<-non_hapax_plot[[i]]
  var_20_mean_temporary_2<- var_20_mean_temporary[var_20_mean_temporary$Freq > quantile(var_10_mean_temporary$Freq, 0.8) & var_20_mean_temporary$Freq <= quantile(var_20_mean_temporary$Freq, 0.9),]
  var_20_mean <- append(var_20_mean, mean(var_20_mean_temporary_2$Freq))
  
}


hapax_plot_def<-vector()
non_hapax_plot_sum<-vector()
token_freq_ex_hapax_sum<-vector()
non_hapax_plot_sum_2<-vector()
non_hapax_plot_sum_3<-vector()
overlap_10_non_hapax<-vector()
union_10_non_hapax<-vector()

for (i in 2:6) {
  hapax_plot_def<-append(hapax_plot_def, length(intersect(as.character(hapax_plot[[i-1]]$Var1), as.character(hapax_plot[[i]]$Var1)))) # number of hapaxes that are both in the second stage and in the first stage (intersect)
  
  non_hapax_plot_sum<-append(non_hapax_plot_sum, sum(non_hapax_plot[[i]]$Freq) - sum(non_hapax_plot[[i-1]]$Freq))  # how many tokens are added to the group of non-hapax types (freq > 1) in the second stage 
  
  zqy<-setdiff(as.character(hapax_plot[[i-1]]$Var1), as.character(hapax_plot[[i]]$Var1)) # list of hapaxes that are in the first stage and not in the list of hapaxes for the second stage
  zqy_2<-non_hapax_plot[[i]][as.character(non_hapax_plot[[i]]$Var1) %in% zqy,]
  token_freq_ex_hapax_sum<-append(token_freq_ex_hapax_sum, sum(zqy_2$Freq)) # sum of token frequences of types that were still hapaxes in the first stage
  non_hapax_plot_sum_2<-append(non_hapax_plot_sum_2, ((sum(non_hapax_plot[[i]]$Freq)- token_freq_ex_hapax_sum[i-1]) - sum(non_hapax_plot[[i-1]]$Freq))) # how many tokens are added to the group of non-hapax types in the second stage, if we restrict ourselves to those types that are non-hapax in both stages
  
  zqw<-non_hapax_plot[[i]][as.character(non_hapax_plot[[i]]$Var1) %in% non_hapax_plot[[i-1]]$Var1,] # all non-hapaxes from the second stage that were also non-hapax in the first stage 
  zqw_2<-sum(non_hapax_plot[[i-1]]$Freq) # sum of token frequencies of non-hapaxes in the first stage
  non_hapax_plot_sum_3<-append(non_hapax_plot_sum_3, sum(zqw$Freq)-zqw_2)
  
  
  var_1<-non_hapax_plot[[i-1]]
  var_1_10<- var_1[var_1$Freq > quantile(var_1$Freq, 0.9),]
  var_2<-non_hapax_plot[[i]]
  var_2_10<- var_2[var_2$Freq > quantile(var_2$Freq, 0.9),]
  
  overlap_10_non_hapax<-append(overlap_10_non_hapax, length(intersect(as.character(var_1_10$Var1), as.character(var_2_10$Var1))))
  union_10_non_hapax<-append(union_10_non_hapax, length(unique(c(as.character(var_1_10$Var1), as.character(var_2_10$Var1)))))
  }


# Overview of tables------------------------------------------------------------------------------------------------------------------------------


# Proportion of hapaxes observed for the second interval that were also hapax in the previous interval (Table 1)
round((hapax_plot_def / number_of_hapaxes[2:6]), 2)

# The type / token ratio for the non-hapaxes (Table 2)
round((number_of_non_hapaxes/number_of_non_hapaxes_tokens),2)  

# The number of tokens that is added, from interval 1 to interval 2, to the non-hapaxes common to both intervals, relative to the token difference between the token counts for non-hapaxes calculated within each interval separately (Table 3)
round((non_hapax_plot_sum_3/non_hapax_plot_sum),2)

# Type / token ratio, restricted to the subset of non-hapaxes, for the 10 % most frequent types vs the rest of the frequency distribution (Table 4)
round(temporary_type_token_10,2) # for all six sample sizes, the type/token ratio for the 10 % most frequent non hapaxes
round(temporary_type_token_90,2) # for all six sample sizes, the type/token ratio for the 90 % non hapaxes with lowest frequency

# The number of types shared between the 10 % most frequent non-hapaxes in each interval, relative to the total number of different types that belong to the 10 % most frequent non-hapaxes in at least one of the two intervals (Table 5)
round((overlap_10_non_hapax / union_10_non_hapax), 2)

# The ratio between (i) the mean of the token frequencies for the interval ]80th ; 90th percentile] and (ii) the mean of the 10 % highest token frequencies (Table 6)
round((var_20_mean / var_10_mean), 2) # to verify whether the ratio between the mean of the token frequencies between 0.8 and 0.9 percentile vs the mean of the 10 % highest token frequencies stays stables across the different periods. The lower the score, the more distance between the top and the subtop.

# Ratio measures with regard to the top of the frequency distribution (Table 7)

# Ratio between rank 1 and rank 2
round(list_dataframes_spec[[1]][1,2] / list_dataframes_spec[[1]][2,2],2)
round(list_dataframes_spec[[2]][1,2] / list_dataframes_spec[[2]][2,2],2)
round(list_dataframes_spec[[3]][1,2] / list_dataframes_spec[[3]][2,2],2)
round(list_dataframes_spec[[4]][1,2] / list_dataframes_spec[[4]][2,2],2)
round(list_dataframes_spec[[5]][1,2] / list_dataframes_spec[[5]][2,2],2)
round(list_dataframes_spec[[6]][1,2] / list_dataframes_spec[[6]][2,2],2)

# Ratio between rank 2 and rank 3
round(list_dataframes_spec[[1]][2,2] / list_dataframes_spec[[1]][3,2],2)
round(list_dataframes_spec[[2]][2,2] / list_dataframes_spec[[2]][3,2],2)
round(list_dataframes_spec[[3]][2,2] / list_dataframes_spec[[3]][3,2],2)
round(list_dataframes_spec[[4]][2,2] / list_dataframes_spec[[4]][3,2],2)
round(list_dataframes_spec[[5]][2,2] / list_dataframes_spec[[5]][3,2],2)
round(list_dataframes_spec[[6]][2,2] / list_dataframes_spec[[6]][3,2],2)

# Ratio between (rank 1 - rank 2) and (rank 2 - rank 3)
round((list_dataframes_spec[[1]][1,2] - list_dataframes_spec[[1]][2,2]) / (list_dataframes_spec[[1]][2,2] - list_dataframes_spec[[1]][3,2]),2)
round((list_dataframes_spec[[2]][1,2] - list_dataframes_spec[[2]][2,2]) / (list_dataframes_spec[[2]][2,2] - list_dataframes_spec[[2]][3,2]),2)
round((list_dataframes_spec[[3]][1,2] - list_dataframes_spec[[3]][2,2]) / (list_dataframes_spec[[3]][2,2] - list_dataframes_spec[[3]][3,2]),2)
round((list_dataframes_spec[[4]][1,2] - list_dataframes_spec[[4]][2,2]) / (list_dataframes_spec[[4]][2,2] - list_dataframes_spec[[4]][3,2]),2)
round((list_dataframes_spec[[5]][1,2] - list_dataframes_spec[[5]][2,2]) / (list_dataframes_spec[[5]][2,2] - list_dataframes_spec[[5]][3,2]),2)
round((list_dataframes_spec[[6]][1,2] - list_dataframes_spec[[6]][2,2]) / (list_dataframes_spec[[6]][2,2] - list_dataframes_spec[[6]][3,2]),2)



#Token difference between first occurrence as hapax and dis legomena / end of interval------------------------------------------------------------

intervals<-c(0,750,1500)

first_rank<-vector()
intervals_rate<-vector()
complements_head_2<-vector()
complements_head_3<-vector()
diff_ranks<-vector()
counter<-0
counter_totaal<-vector()

tokens<-select(final_dataset, SUBJECT_COMPLEMENT_TYPE)
tokens$VERB<-rep("raken", 1500)

for (i in 2:length(intervals)) { 
  
  a<-tokens[(intervals[i-1]+1):intervals[i],] 
  
  for (j in 2:length(a[,1])) {
    
    if ((a$SUBJECT_COMPLEMENT_TYPE[j] %in% complements_head_2) | (a$SUBJECT_COMPLEMENT_TYPE[j] %in% complements_head_3)) {next}  
    
    else if (a$SUBJECT_COMPLEMENT_TYPE[j] %in% a$SUBJECT_COMPLEMENT_TYPE[1:(j-1)]) {
      first_rank<-min(which(a$SUBJECT_COMPLEMENT_TYPE[j] == a$SUBJECT_COMPLEMENT_TYPE[1:(j-1)]))
      intervals_rate<-append(intervals_rate, intervals[i])
      complements_head_2<-append(complements_head_2, as.character(a$SUBJECT_COMPLEMENT_TYPE[j]))
      diff_ranks<-append(diff_ranks, j-first_rank)
      counter<-counter+1
    }
    
    else if (!(a$SUBJECT_COMPLEMENT_TYPE[j] %in% a$SUBJECT_COMPLEMENT_TYPE[(j+1):intervals[i]])) {
      
      complements_head_3<-append(complements_head_3, as.character(a$SUBJECT_COMPLEMENT_TYPE[j]))
    }}
  
  counter_totaal<-append(counter_totaal, 1:counter)
  counter<-0}
V1_rate<-data.frame(counter_totaal, intervals_rate, complements_head_2, diff_ranks)



first_rank<-vector()
intervals_rate<-vector()
diff_ranks<-vector()
hapax_without_end_total<-vector()

for (i in 2:length(intervals)) { 
  
  a<-tokens[(intervals[i-1]+1):intervals[i],]
  
  hapax_without_end_1 <-as.data.frame(table(a))
  hapax_without_end_2<-filter(hapax_without_end_1, Freq==1)
  hapax_without_end_3<-as.character(hapax_without_end_2$SUBJECT_COMPLEMENT_TYPE)
  hapax_without_end_total<-append(hapax_without_end_total, hapax_without_end_3)
  
  for (j in 1:length(hapax_without_end_3)) {
    
    first_rank<-which(hapax_without_end_3[j] == a$SUBJECT_COMPLEMENT_TYPE) 
    intervals_rate<-append(intervals_rate, intervals[i])
    diff_ranks<-append(diff_ranks, 750-first_rank)  }}
V1_rate_2<-data.frame(intervals_rate, hapax_without_end_total, diff_ranks)  # doubles are to be removed

delete<-vector()
for (i in 2:length(V1_rate_2[,1])) {
  if (V1_rate_2$hapax_without_end_total[i] %in% V1_rate_2$hapax_without_end_total[1:(i-1)]) {
    delete<-append(delete, i)
  }}
V1_rate_2<-V1_rate_2[-delete,]

delete_2<-ifelse(V1_rate_2$hapax_without_end_total %in% complements_head_2, TRUE, FALSE)
V1_rate_2<-V1_rate_2[delete_2==FALSE,]

# cf. Figures 2 and 3 in the associated article 
png(file = "Boxplot_1.png")
ggplot(data=V1_rate, aes(y = diff_ranks, x = factor(intervals_rate))) + geom_boxplot() + geom_dotplot(binaxis='y', stackdir='center', dotsize=0.5, binwidth = 20) + theme_classic() + labs(x="intervals", y = "token difference between first occurrence as hapax and conversion to dis legomenon") + scale_x_discrete(labels = c('[1 ; 750]','[751 ; 1500]'))
dev.off()

png(file = "Boxplot_2.png")
ggplot(data=V1_rate_2, aes(y = diff_ranks, x = factor(intervals_rate))) + geom_boxplot() + geom_dotplot(binaxis='y', stackdir='center', dotsize=0.5, binwidth = 20) + theme_classic() + labs(x="intervals", y = "token difference between first occurrence as hapax and end of interval") +  
  scale_x_discrete(labels = c('[1 ; 750]','[751 ; 1500]'))
dev.off()

table(V1_rate$intervals_rate)
table(V1_rate_2$intervals_rate)


# remove memory---------------------------------------------------------------------------------------------------------------------------------
rm(list=ls(all=TRUE))
